@@ -41,8 +41,7 @@ struct SimpleScanPrivate
     GtkWidget *scan_button_label, *continuous_scan_button_label, *page_label;
     GtkWidget *device_combo, *mode_combo;
     GtkTreeModel *device_model, *mode_model;
-    GtkWidget *preview_area;
-    GtkWidget *zoom_scale;
+    GtkWidget *preview_box, *preview_area, *preview_scroll;
     GtkWidget *page_delete_menuitem, *crop_rotate_menuitem;
 
     GtkWidget *authorize_dialog;
@@ -52,8 +51,11 @@ struct SimpleScanPrivate
     GtkWidget *preferences_dialog;
     GtkWidget *replace_pages_check;
 
+    Book *book;
     BookView *book_view;
     gboolean updating_page_menu;
+    Orientation default_orientation;
+    gboolean book_is_placeholder; // FIXME: Needs to be cleared when scan starts
 
     gchar *default_file_name;
     gboolean scanning;
@@ -64,6 +66,19 @@ struct SimpleScanPrivate
 
 G_DEFINE_TYPE (SimpleScan, ui, G_TYPE_OBJECT);
 
+static struct
+{
+   const gchar *key;
+   Orientation orientation;
+} orientation_keys[] = 
+{
+  { "top-to-bottom", TOP_TO_BOTTOM },
+  { "bottom-to-top", BOTTOM_TO_TOP },
+  { "left-to-right", LEFT_TO_RIGHT },
+  { "right-to-left", RIGHT_TO_LEFT },
+  { NULL, 0 }
+};
+
 
 static gboolean
 find_scan_device (SimpleScan *ui, const char *device, GtkTreeIter *iter)
@@ -178,6 +193,31 @@ ui_set_selected_device (SimpleScan *ui, const gchar *device)
 }
 
 
+static void
+add_default_page (SimpleScan *ui)
+{
+    if (book_get_n_pages (ui->priv->book) > 0)
+        return;
+
+    /* Start with A4 white image at 72dpi */
+    // FIXME: Remember last page dimensions
+    book_append_page (ui->priv->book, 595, 842, 72, ui->priv->default_orientation);
+
+    /* Remove this page on the next scan */
+    ui->priv->book_is_placeholder = TRUE;
+}
+
+
+void new_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+new_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    book_clear (ui->priv->book);
+    add_default_page (ui);
+}
+
+
 static void
 set_document_hint (SimpleScan *ui, const gchar *document_hint)
 {
@@ -217,7 +257,7 @@ get_document_hint (SimpleScan *ui)
 static gboolean
 get_replace_pages (SimpleScan *ui)
 {
-    return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ui->priv->replace_pages_check));
+    return ui->priv->book_is_placeholder || gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ui->priv->replace_pages_check));
 }
 
 
@@ -344,9 +384,13 @@ G_MODULE_EXPORT
 void
 rotate_left_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 {
+    Page *page;
+
     if (ui->priv->updating_page_menu)
         return;
-    page_rotate_left (book_view_get_selected (ui->priv->book_view));
+    page = book_view_get_selected (ui->priv->book_view);
+    page_rotate_left (page);
+    ui->priv->default_orientation = page_get_orientation (page);
 }
 
 
@@ -355,9 +399,13 @@ G_MODULE_EXPORT
 void
 rotate_right_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 {
+    Page *page;
+
     if (ui->priv->updating_page_menu)
         return;
-    page_rotate_right (book_view_get_selected (ui->priv->book_view));    
+    page = book_view_get_selected (ui->priv->book_view);
+    page_rotate_right (page);
+    ui->priv->default_orientation = page_get_orientation (page);
 }
 
 
@@ -528,15 +576,13 @@ draw_page (GtkPrintOperation *operation,
            SimpleScan        *ui)
 {
     cairo_t *context;
-    Book *book;
     Page *page;
     GdkPixbuf *image;
     gboolean is_landscape = FALSE;
 
     context = gtk_print_context_get_cairo_context (print_context);
    
-    book = book_view_get_book (ui->priv->book_view);
-    page = book_get_page (book, page_number);
+    page = book_get_page (ui->priv->book, page_number);
 
     /* Rotate to same aspect */
     if (gtk_print_context_get_width (print_context) > gtk_print_context_get_height (print_context))
@@ -575,12 +621,9 @@ print_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
     GtkPrintOperation *print;
     GtkPrintOperationResult result;
     GError *error = NULL;
-    Book *book;
    
-    book = book_view_get_book (ui->priv->book_view);
-
     print = gtk_print_operation_new ();
-    gtk_print_operation_set_n_pages (print, book_get_n_pages (book));
+    gtk_print_operation_set_n_pages (print, book_get_n_pages (ui->priv->book));
     g_signal_connect (print, "draw-page", G_CALLBACK (draw_page), ui);
 
     result = gtk_print_operation_run (print, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
@@ -590,24 +633,6 @@ print_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 }
 
 
-void next_page_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
-G_MODULE_EXPORT
-void
-next_page_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
-{
-    book_view_select_next_page (ui->priv->book_view);
-}
-
-
-void prev_page_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
-G_MODULE_EXPORT
-void
-prev_page_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
-{
-    book_view_select_prev_page (ui->priv->book_view);    
-}
-
-
 static void
 load_device_cache (SimpleScan *ui)
 {
@@ -757,9 +782,11 @@ about_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
                            "version", VERSION,
                            "comments", description,
                            "authors", authors,
+                           "translator-credits", _("translator-credits"),
                            "website", "https://launchpad.net/simple-scan",
                            "copyright", "Copyright (C) 2009 Canonical Ltd.",
                            "license", license,
+                           "wrap-license", TRUE,
                            NULL);
 }
 
@@ -768,6 +795,7 @@ static void
 quit (SimpleScan *ui)
 {
     char *device, *document_type;
+    gint i;
 
     save_device_cache (ui);
 
@@ -782,11 +810,15 @@ quit (SimpleScan *ui)
     g_free (document_type);
 
     gconf_client_set_bool(ui->priv->client, "/apps/simple-scan/replace_pages",
-                          get_replace_pages (ui), NULL);
+                          gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ui->priv->replace_pages_check)), NULL);
 
     gconf_client_set_int(ui->priv->client, "/apps/simple-scan/window_width", ui->priv->window_width, NULL);
     gconf_client_set_int(ui->priv->client, "/apps/simple-scan/window_height", ui->priv->window_height, NULL);
     gconf_client_set_bool(ui->priv->client, "/apps/simple-scan/window_is_maximized", ui->priv->window_is_maximized, NULL);
+
+    for (i = 0; orientation_keys[i].key != NULL && orientation_keys[i].orientation != ui->priv->default_orientation; i++);
+    if (orientation_keys[i].key != NULL)
+        gconf_client_set_string(ui->priv->client, "/apps/simple-scan/scan_direction", orientation_keys[i].key, NULL);
    
     g_signal_emit (G_OBJECT (ui), signals[QUIT], 0);
 }
@@ -836,13 +868,29 @@ window_delete_event_cb (GtkWidget *widget, GdkEvent *event, SimpleScan *ui)
 }
 
 
+static void
+page_removed_cb (Book *book, Page *page, SimpleScan *ui)
+{
+    /* Ensure always one page */
+    add_default_page (ui);
+}
+
+
+static void
+book_cleared_cb (Book *book, SimpleScan *ui)
+{
+    /* Must have been cleared for next scan */
+    ui->priv->book_is_placeholder = FALSE;
+}
+
+
 static void
 ui_load (SimpleScan *ui)
 {
     GtkBuilder *builder;
     GError *error = NULL;
     GtkCellRenderer *renderer;
-    gchar *device, *document_type;
+    gchar *device, *document_type, *scan_direction;
     gboolean replace_pages;
 
     builder = ui->priv->builder = gtk_builder_new ();
@@ -867,8 +915,9 @@ ui_load (SimpleScan *ui)
     ui->priv->device_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->device_combo));
     ui->priv->mode_combo = GTK_WIDGET (gtk_builder_get_object (builder, "mode_combo"));
     ui->priv->mode_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->mode_combo));
+    ui->priv->preview_box = GTK_WIDGET (gtk_builder_get_object (builder, "preview_vbox"));
     ui->priv->preview_area = GTK_WIDGET (gtk_builder_get_object (builder, "preview_area"));
-    ui->priv->zoom_scale = GTK_WIDGET (gtk_builder_get_object (builder, "zoom_scale"));
+    ui->priv->preview_scroll = GTK_WIDGET (gtk_builder_get_object (builder, "preview_scrollbar"));
     ui->priv->page_delete_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "page_delete_menuitem"));
     ui->priv->crop_rotate_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "crop_rotate_menuitem"));
 
@@ -910,10 +959,20 @@ ui_load (SimpleScan *ui)
     
     ui->priv->book_view = book_view_new ();
     g_signal_connect (ui->priv->book_view, "page-selected", G_CALLBACK (page_selected_cb), ui);
-    book_view_set_widget (ui->priv->book_view, ui->priv->preview_area,
-                          GTK_WIDGET (gtk_builder_get_object (builder, "page_menu")));
-    gtk_range_set_adjustment (GTK_RANGE (ui->priv->zoom_scale),
-                              book_view_get_zoom_adjustment (ui->priv->book_view));
+    book_view_set_widgets (ui->priv->book_view,
+                           ui->priv->preview_box,
+                           ui->priv->preview_area,
+                           ui->priv->preview_scroll,
+                           GTK_WIDGET (gtk_builder_get_object (builder, "page_menu")));
+
+    /* Find default scan direction */
+    scan_direction = gconf_client_get_string(ui->priv->client, "/apps/simple-scan/scan_direction", NULL);
+    if (scan_direction) {
+        gint i;
+        for (i = 0; orientation_keys[i].key != NULL && strcmp (orientation_keys[i].key, scan_direction) != 0; i++);
+        if (orientation_keys[i].key != NULL)
+            ui->priv->default_orientation = orientation_keys[i].orientation;
+    }
 
     /* Restore window size */
     ui->priv->window_width = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/window_width", NULL);
@@ -929,6 +988,9 @@ ui_load (SimpleScan *ui)
         g_debug ("Restoring window to maximized");
         gtk_window_maximize (GTK_WINDOW (ui->priv->window));
     }
+
+    add_default_page (ui);
+    book_view_set_book (ui->priv->book_view, ui->priv->book);
 }
 
 
@@ -939,10 +1001,10 @@ ui_new ()
 }
 
 
-void
-ui_set_book (SimpleScan *ui, Book *book)
+Book *
+ui_get_book (SimpleScan *ui)
 {
-    book_view_set_book (ui->priv->book_view, book);
+    return ui->priv->book;
 }
 
 
@@ -1105,10 +1167,14 @@ ui_init (SimpleScan *ui)
 {
     ui->priv = G_TYPE_INSTANCE_GET_PRIVATE (ui, SIMPLE_SCAN_TYPE, SimpleScanPrivate);
 
+    ui->priv->book = book_new ();
+    g_signal_connect (ui->priv->book, "page-removed", G_CALLBACK (page_removed_cb), ui);
+    g_signal_connect (ui->priv->book, "cleared", G_CALLBACK (book_cleared_cb), ui);
+   
     ui->priv->client = gconf_client_get_default();
     gconf_client_add_dir(ui->priv->client, "/apps/simple-scan", GCONF_CLIENT_PRELOAD_NONE, NULL);
 
     ui->priv->default_file_name = g_strdup (_("Scanned Document.pdf"));
     ui->priv->scanning = FALSE;
-    ui_load (ui);
+    ui_load (ui);   
 }
